import 'package:freezed_annotation/freezed_annotation.dart';
import 'package:fhir/primitive_types/primitive_types.dart';

import '../../../search_parameter_types/search_parameter_types.dart';
import '../../../search_parameters.dart';

part 'individuals.freezed.dart';
part 'individuals.g.dart';

@freezed
abstract class GroupSearch with Stu3SearchParameters implements _$GroupSearch {
  factory GroupSearch({
    List<Id> searchId,
    List<SearchDate> searchLastUpdated,
    List<SearchToken> searchTag,
    List<SearchUri> searchProfile,
    List<SearchToken> searchSecurity,
    List<SearchString> searchText,
    List<SearchString> searchContent,
    List<Id> searchList,
//List<SearchString> searchHas,
//List<SearchToken> searchType,
    List<SearchToken> actual,
    List<SearchToken> characteristic,
    @JsonKey(name: 'characteristic-value')
        List<SearchComposite> characteristicValue,
    List<SearchToken> code,
    List<SearchToken> exclude,
    List<SearchToken> identifier,
    List<SearchReference> member,
    List<SearchToken> type,
    List<SearchToken> value,
  }) = _GroupSearch;

  GroupSearch._();
  factory GroupSearch.fromJson(Map<String, dynamic> json) =>
      _$GroupSearchFromJson(json);
}

@freezed
abstract class PatientSearch
    with Stu3SearchParameters
    implements _$PatientSearch {
  factory PatientSearch({
    List<Id> searchId,
    List<SearchDate> searchLastUpdated,
    List<SearchToken> searchTag,
    List<SearchUri> searchProfile,
    List<SearchToken> searchSecurity,
    List<SearchString> searchText,
    List<SearchString> searchContent,
    List<Id> searchList,
//List<SearchString> searchHas,
//List<SearchToken> searchType,
    List<SearchToken> active,
    List<SearchString> address,
    @JsonKey(name: 'address-city') List<SearchString> addressCity,
    @JsonKey(name: 'address-country') List<SearchString> addressCountry,
    @JsonKey(name: 'address-postalcode') List<SearchString> addressPostalcode,
    @JsonKey(name: 'address-state') List<SearchString> addressState,
    @JsonKey(name: 'address-use') List<SearchToken> addressUse,
    @JsonKey(name: 'animal-breed') List<SearchToken> animalBreed,
    @JsonKey(name: 'animal-species') List<SearchToken> animalSpecies,
    List<SearchDate> birthdate,
    @JsonKey(name: 'death-date') List<SearchDate> deathDate,
    List<SearchToken> deceased,
    List<SearchToken> email,
    List<SearchString> family,
    List<SearchToken> gender,
    @JsonKey(name: 'general-practitioner')
        List<SearchReference> generalPractitioner,
    List<SearchString> given,
    List<SearchToken> identifier,
    List<SearchToken> language,
    List<SearchReference> link,
    List<SearchString> name,
    List<SearchReference> organization,
    List<SearchToken> phone,
    List<SearchString> phonetic,
    List<SearchToken> telecom,
  }) = _PatientSearch;

  PatientSearch._();
  factory PatientSearch.fromJson(Map<String, dynamic> json) =>
      _$PatientSearchFromJson(json);
}

@freezed
abstract class RelatedPersonSearch
    with Stu3SearchParameters
    implements _$RelatedPersonSearch {
  factory RelatedPersonSearch({
    List<Id> searchId,
    List<SearchDate> searchLastUpdated,
    List<SearchToken> searchTag,
    List<SearchUri> searchProfile,
    List<SearchToken> searchSecurity,
    List<SearchString> searchText,
    List<SearchString> searchContent,
    List<Id> searchList,
//List<SearchString> searchHas,
//List<SearchToken> searchType,
    List<SearchString> address,
    @JsonKey(name: 'address-city') List<SearchString> addressCity,
    @JsonKey(name: 'address-country') List<SearchString> addressCountry,
    @JsonKey(name: 'address-postalcode') List<SearchString> addressPostalcode,
    @JsonKey(name: 'address-state') List<SearchString> addressState,
    @JsonKey(name: 'address-use') List<SearchToken> addressUse,
    List<SearchDate> birthdate,
    List<SearchToken> email,
    List<SearchToken> gender,
    List<SearchToken> phone,
    List<SearchString> phonetic,
    List<SearchToken> telecom,
    List<SearchToken> active,
    List<SearchToken> identifier,
    List<SearchString> name,
    List<SearchReference> patient,
  }) = _RelatedPersonSearch;

  RelatedPersonSearch._();
  factory RelatedPersonSearch.fromJson(Map<String, dynamic> json) =>
      _$RelatedPersonSearchFromJson(json);
}

@freezed
abstract class PractitionerSearch
    with Stu3SearchParameters
    implements _$PractitionerSearch {
  factory PractitionerSearch({
    List<Id> searchId,
    List<SearchDate> searchLastUpdated,
    List<SearchToken> searchTag,
    List<SearchUri> searchProfile,
    List<SearchToken> searchSecurity,
    List<SearchString> searchText,
    List<SearchString> searchContent,
    List<Id> searchList,
//List<SearchString> searchHas,
//List<SearchToken> searchType,
    List<SearchString> address,
    @JsonKey(name: 'address-city') List<SearchString> addressCity,
    @JsonKey(name: 'address-country') List<SearchString> addressCountry,
    @JsonKey(name: 'address-postalcode') List<SearchString> addressPostalcode,
    @JsonKey(name: 'address-state') List<SearchString> addressState,
    @JsonKey(name: 'address-use') List<SearchToken> addressUse,
    List<SearchToken> email,
    List<SearchString> family,
    List<SearchToken> gender,
    List<SearchString> given,
    List<SearchToken> phone,
    List<SearchString> phonetic,
    List<SearchToken> telecom,
    List<SearchToken> active,
    List<SearchToken> communication,
    List<SearchToken> identifier,
    List<SearchString> name,
  }) = _PractitionerSearch;

  PractitionerSearch._();
  factory PractitionerSearch.fromJson(Map<String, dynamic> json) =>
      _$PractitionerSearchFromJson(json);
}

@freezed
abstract class PersonSearch
    with Stu3SearchParameters
    implements _$PersonSearch {
  factory PersonSearch({
    List<Id> searchId,
    List<SearchDate> searchLastUpdated,
    List<SearchToken> searchTag,
    List<SearchUri> searchProfile,
    List<SearchToken> searchSecurity,
    List<SearchString> searchText,
    List<SearchString> searchContent,
    List<Id> searchList,
//List<SearchString> searchHas,
//List<SearchToken> searchType,
    List<SearchString> address,
    @JsonKey(name: 'address-city') List<SearchString> addressCity,
    @JsonKey(name: 'address-country') List<SearchString> addressCountry,
    @JsonKey(name: 'address-postalcode') List<SearchString> addressPostalcode,
    @JsonKey(name: 'address-state') List<SearchString> addressState,
    @JsonKey(name: 'address-use') List<SearchToken> addressUse,
    List<SearchDate> birthdate,
    List<SearchToken> email,
    List<SearchToken> gender,
    List<SearchToken> phone,
    List<SearchString> phonetic,
    List<SearchToken> telecom,
    List<SearchToken> identifier,
    List<SearchReference> link,
    List<SearchString> name,
    List<SearchReference> organization,
    List<SearchReference> patient,
    List<SearchReference> practitioner,
    List<SearchReference> relatedperson,
  }) = _PersonSearch;

  PersonSearch._();
  factory PersonSearch.fromJson(Map<String, dynamic> json) =>
      _$PersonSearchFromJson(json);
}

@freezed
abstract class PractitionerRoleSearch
    with Stu3SearchParameters
    implements _$PractitionerRoleSearch {
  factory PractitionerRoleSearch({
    List<Id> searchId,
    List<SearchDate> searchLastUpdated,
    List<SearchToken> searchTag,
    List<SearchUri> searchProfile,
    List<SearchToken> searchSecurity,
    List<SearchString> searchText,
    List<SearchString> searchContent,
    List<Id> searchList,
//List<SearchString> searchHas,
//List<SearchToken> searchType,
    List<SearchToken> email,
    List<SearchToken> phone,
    List<SearchToken> telecom,
    List<SearchToken> active,
    List<SearchDate> date,
    List<SearchReference> endpoint,
    List<SearchToken> identifier,
    List<SearchReference> location,
    List<SearchReference> organization,
    List<SearchReference> practitioner,
    List<SearchToken> role,
    List<SearchReference> service,
    List<SearchToken> specialty,
  }) = _PractitionerRoleSearch;

  PractitionerRoleSearch._();
  factory PractitionerRoleSearch.fromJson(Map<String, dynamic> json) =>
      _$PractitionerRoleSearchFromJson(json);
}
